/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 70 (Table 1.20)
**
*/

new;
library rpb,pgraph;
cls;

__output = 0;

let mu =    0.05 0.06 0.08 0.06;
let _sigma_ = 0.15 0.20 0.25 0.30;
n = rows(_Sigma_);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

Sigma = rho .* _sigma_ .* _sigma_';

mu = sqrt(diag(Sigma));

print ftosa(100*(_sigma_~rho),"%lf",5,2);
print "--------------------------------------------------------------------------------------------------------";

A = ones(1,n); B = 1; e = ones(n,1);

Bounds = e .* (0.10~0.40);

sv = ones(n,1)/n;
{x_star,mu_star,sigma_star,SR_star,retcode} = Compute_MSR_Portfolio(sv,Sigma,0);
vol = sqrt(x_star'Sigma*x_star);
{y_star,mu_y,sigma_y,gamma_y,retcode} = Compute_MVO_Portfolio(mu,Sigma,A,B,0,0,0,vol,2);
{z_star,mu_z,sigma_z,gamma_z,retcode} = Compute_MVO_Portfolio(mu,Sigma,A,B,0,0,0,gamma_y,0);

results = x_star~y_star~z_star;
print ftosa(100*results,"%lf",5,3);
print "--------------------------------------------------------------------------------------------------------";

{x_tilde,mu_tilde,sigma_tilde,SR_tilde,retcode} = Compute_MSR_Portfolio(sv,Sigma,Bounds);
vol_tilde = sqrt(x_tilde'Sigma*x_tilde);
{y_tilde,mu_tilde,sigma_tilde,gamma_tilde,retcode} = Compute_MVO_Portfolio(mu,Sigma,A,B,0,0,Bounds,vol_tilde,2);
{z_tilde,u1,u2,lambda1,lambda2,retcode} = QProg(ones(n,1)/n,Sigma,gamma_tilde*mu,A,B,0,0,Bounds);
lambda = lambda1~lambda2;
Sigma_tilde = Sigma - (lambda1 + lambda1') + (lambda2 + lambda2');

{x_bar,u1,u2,lambda1,lambda2,retcode} = QProg(ones(n,1)/n,Sigma_tilde,gamma_tilde*mu,A,B,0,0,0);

results = x_tilde~y_tilde~z_tilde~x_bar;
print ftosa(100*results,"%lf",5,3);
print "--------------------------------------------------------------------------------------------------------";

_sigma_tilde = sqrt(diag(Sigma_tilde));
rho_tilde = Sigma_tilde ./ _sigma_tilde ./ _sigma_tilde';

print ftosa(100*(x_tilde~lambda),"%lf",5,3)$~ftosa(100*(_sigma_tilde~rho_tilde),"%lf",5,2);
print "--------------------------------------------------------------------------------------------------------";

sh = 0.5 * mu ./ _sigma_;
sh_tilde = 0.5 * mu ./ _sigma_tilde;

print ftosa(sh~sh_tilde,"%lf",5,2);

