/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 86 and 87
**
*/

new;
library rpb,pgraph;
rpbSet;

cls;

let mu = 0.10 0.15 0.20;
let sigma = 0.30 0.20 0.15;
let rho = 1.00
          0.80 1.00
          0.50 0.30 1.00;
rho = xpnd(rho);
cov = rho .* sigma .* sigma';

let x = 0.50 0.20 0.30;

alpha = 0.99;
{Vol,mr,rc,prc} = Compute_RC_Vol(x,cov);
{VaR,mr,rc,prc} = Compute_RC_VaR(x,mu,cov,alpha);
{ES,mr,rc,prc} = Compute_RC_ES(x,mu,cov,alpha);

declare matrix mu_u,mu_v,sigma_u,sigma_v,rho_uv,v0;

proc f(u,v);
  local pdf;

  pdf = 1 / (2 * pi * sigma_u * sigma_v * sqrt(1 - rho_uv^2) ) * 
        exp( -1 / (2 * (1 - rho_uv^2)) * ( (u - mu_u)^2 / (sigma_u^2) + (v - mu_v)^2 / (sigma_v^2) - 2 * rho_uv * (u - mu_u) .* (v - mu_v) ./ (sigma_u .* sigma_v) ) );
 
  retp(pdf);
endp;

proc g1(u,v);
  local e;
  e = (v .<= -v0);
  retp( u .* e .* f(u,v) );
endp;

proc g2(u,v);
  retp( u .* f(u,v) );
endp;

E = zeros(3,2);

infinity = 10;

i = 1;
do until i > 3;
  mu_u = mu[i];
  mu_v = mu'x;
  sigma_u = sigma[i];
  sigma_v = Vol;
  rho_uv = submat(cov*x,i,1)/(sigma_u * sigma_v);
  v0 = VaR;
  ul = mu_u + (infinity|-infinity) * sigma_u;
  vl = mu_v + (infinity|-infinity) * sigma_v;
  E[i,1] = quadLegendre2(&g1,ul,vl,256);
  vl[1] = -v0;
  E[i,2] = quadLegendre2(&g2,ul,vl,256);
  i = i + 1;
endo;

rc1 = - x / (1-alpha) .* E[.,1];
rc2 = - x / (1-alpha) .* E[.,2];

print ftosa(100*(rc~rc1~rc2),"%lf",3,4);


