/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 91 (Remark 24)
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123;

rho = 0.5;

mu1 = 0.10;
sigma1 = 0.20;
w1 = 0.50;

mu2 = 0.10;
sigma2 = 0.25;
w2 = 0.50;

w = w1|w2;

alpha = 0.90;
nS = 5000;
nR = 2000;
indx_alpha = nS * alpha;

h = 2.5/100;
h = h * nS;
indx = seqa(indx_alpha-h,1,2*h+1);
p = zeros(nS,1);
p[indx] = ones(2*h+1,1)/(2*h+1);

SD = zeros(nR,1);
RC_SD = zeros(2,nR);
VaR = zeros(nR,1);
RC_VaR = zeros(2,nR);
VaR_w = zeros(nR,1);
RC_VaR_w = zeros(2,nR);
ES = zeros(nR,1);
RC_ES = zeros(2,nR);
ES_w = zeros(nR,1);
RC_ES_w = zeros(2,nR);

i = 1;
do until i > nR;
  {u1,u2} = rndCopulaNormal2(rho,ns);

  R1 = mu1 + sigma1 * cdfni(u1);
  R2 = mu2 + sigma2 * cdfni(u2);

  cov = vcx(R1~R2);
  SD[i] = sqrt(w'cov*w);
  RC_SD[.,i] = w .* (cov*w)/sqrt(w'cov*w);

  L1 = -w1 * R1;
  L2 = -w2 * R2;

  L = L1~L2;

  {VaR[i],RC_VaR[.,i],ES[i],RC_ES[.,i]} = Compute_Empirical_RC(L,alpha);
  {VaR_w[i],RC_VaR_w[.,i],ES_w[i],RC_ES_w[.,i]} = Compute_Empirical_RC(L,p);

  i = i + 1;
endo;

RC_SD = RC_SD';
RC_VaR = RC_VaR';
RC_ES = RC_ES';
RC_VaR_w = RC_VaR_w';

results = ((meanc(SD)|stdc(SD))                     |
          (meanc(RC_SD[.,1])|stdc(RC_SD[.,1]))     |
          (meanc(RC_SD[.,2])|stdc(RC_SD[.,2]))     ) ~
          ((meanc(VaR)|stdc(VaR))                     |
          (meanc(RC_VaR[.,1])|stdc(RC_VaR[.,1]))     |
          (meanc(RC_VaR[.,2])|stdc(RC_VaR[.,2]))     )~
          ((meanc(ES)|stdc(ES))                     |
          (meanc(RC_ES[.,1])|stdc(RC_ES[.,1]))     |
          (meanc(RC_ES[.,2])|stdc(RC_ES[.,2]))     )~
          ((meanc(VaR_w)|stdc(VaR_w))                     |
          (meanc(RC_VaR_w[.,1])|stdc(RC_VaR_w[.,1]))     |
          (meanc(RC_VaR_w[.,2])|stdc(RC_VaR_w[.,2]))     );
print ftosa(100*results,"%lf",3,2);

results = results[3 5,.]./results[1,.];
print ftosa(100*results,"%lf",3,2);
print "--------------------------------------------------------------------";

mu = w1 * mu1 + w2 * mu2;
c_VaR = (meanc(VaR) + mu)/meanc(SD);
c_ES = (meanc(ES) + mu)/meanc(SD);
kappa = c_ES/c_VaR;
print ftosa(c_VaR~c_ES~kappa,"%lf",3,4);

c_Var_Normal = cdfni(alpha);
c_ES_Normal = pdfn(cdfni(alpha))/(1-alpha);
kappa_Normal = c_ES_Normal/c_VaR_Normal;
print ftosa(c_VaR_Normal~c_ES_Normal~kappa_Normal,"%lf",3,4);  