/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 146 (Table 2.35)
**
*/

new;

#define use_co 1

#ifdef use_co
  library rpb,pgraph,co;
#else
  library rpb,pgraph;
#endif

cls;

Let A[4,3] = 0.9    0.0     0.5
             1.1    0.5     0.0
             1.2    0.3     0.2
             0.8    0.1     0.7;

let vol_F = 0.20 0.10 0.10;
Omega = diagrv(eye(3),vol_F^2);
let sigma_A = 0.10 0.15 0.10 0.15;
D = diagrv(eye(4),sigma_A^2);

Sigma = A*Omega*A' + D;
vol_A = sqrt(diag(Sigma));
rho_A = Sigma ./ vol_A ./ vol_A';

n = rows(A);
m = cols(A);

print ftosa(100*vol_A,"%lf",3,2);
print ftosa(100*rho_A,"%lf",3,2);

indx = seqa(1,1,3);

_rpb_factor_bounds = 0.10~1;

x = zeros(n,3);
y = zeros(n,3);
f_mr = zeros(n,3);
f_rc = zeros(n,3);
f_prc = zeros(n,3);

i = 1;
do until i > 3;
  {x[.,i],y[.,i],f_mr[.,i],f_rc[.,i],f_prc[.,i]} = Compute_RB_Factor(0,Sigma,A,Omega,D,0,indx,i+1);
  i = i + 1;
endo;

cls;

H_star = zeros(3,1);
G = zeros(3,1);
I_star = zeros(3,1);

j = 1;
do until j > 3;
  {vol,mr,rc,prc} = Compute_Risk_Contribution(x[.,j],Sigma);
  p = f_rc[1:m,j];
  p = p/sumc(p);
  {H,H_star[j],N_eff} = Herfindahl_Index(p);
  {G[j],G_star} = Gini_Index(p);
  {I,I_star[j]} = Entropy_Index(p);

  print "------------------------------------------------------";
  print ftos(100*vol,"vol = %lf",3,2);
  print ftosa(100*(y[.,i]~f_mr[.,i]~f_rc[.,i]~f_prc[.,i]),"%lf",3,2);
  print ftos(100*vol,"vol = %lf",3,2);
  print ftosa(100*(x[.,i]~mr~rc~prc),"%lf",3,2);
  print;
  print ftosa(100*(H_star[j]~G[j]~I_star[j]),"%lf",3,2);
  
  j = j + 1;
endo;

print "------------------------------------------------------";

Results = 100*x;
print ftosa(Results,"%lf",3,2);
Results = H_star' | G' | I_star';
print ftosa(Results,"%lf",3,4);

