/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 225 & 226 (Tables 4.13 & 4.14)
**
*/

new;
library rpb,pgraph;

cls;

FileName = "Data//sovereign-bonds.fmt";
loadm databuf = ^FileName;
Dates = vread(databuf,"dates");
Spread = vread(databuf,"spread");
Spread_Greece = vread(databuf,"sp-gr");
Duration = vread(databuf,"duration");
Weights_EGBI = vread(databuf,"w(egbi)");
Weights_GDP = vread(databuf,"w(gdp)");
Weights_DEBT = vread(databuf,"w(debt)");
EGBI_index = vread(databuf,"index");
EGBI_Subindex = vread(databuf,"subindex");

Begin_Date = 20071231;
End_Date = 20120630;
nDates = rows(Dates);

indx_Begin_Date = closest_index(Begin_Date,Dates);

nCountries = cols(Spread);
nLags = 260;
sigma_S = Rolling_Volatility(Dates,Spread,nLags);
sigma_S = missex(sigma_S,Dates .< Begin_Date);

// Correction for Greek Spread

indx = indnv(20110914,dates);
i = indx + 1;
do until i > nDates;
  {t,S} = packr2(Dates[i-nLags:i],Spread_Greece[i-nLags:i]);  
   dt = etdays2(lag1(t),t);
   dt = _max_(dt - 2,1.0);
   R = (ln(S) - lag1(ln(S))) ./ dt;
   x = packr(R);
   sigma_S[i,6] = sqrt(260) * stdc(x);
   i = i + 1;
endo;

{cov,vol,cor} = Rolling_Covariance(Dates,Spread,nLags,1);
vol[.,6] = sigma_S[.,6];

// Correction for Greek Spread
cor2 = cor;
tmp_Spread = Spread;
tmp_Spread[.,6] = Spread_Greece;
indx = indnv(20110914,dates);
i = indx + 1;
do until i > nDates;
  {t,S} = packr2(Dates[i-nLags:i],Spread[i-nLags:i,.]);  
   dt = etdays2(lag1(t),t);
   dt = _max_(dt - 2,1.0);
   R = (ln(S) - lag1(ln(S))) ./ dt;
   x = packr(R);
   c = corrx(x); 
   cor2[i,6,.] = c[6,.];
   cor2[i,.,6] = c[.,6];
   i = i + 1;
endo;

let user_dates = 20080630 20090630 20100630 20110630 20120630;
let user_dates = 20080101 20090101 20100101 20110101 20120101;
indx = closest_index(user_dates,Dates);
user_dates = Dates[indx];
data = 1e4*(Spread[indx,.]'|median(Spread[indx,.]')');
print ftosa(data,"%lf",3,0);

data = 1e2*(vol[indx,.]'|meanc(vol[indx,.]')');
print ftosa(data,"%lf",3,1);

m = meanc(vol');
call dstat(0,packr(m));
print;

cor = cor2;

user_date = 20080103;
indx = indnv(user_date,Dates);
data = (1e4*Spread[indx,.])'~1e2*vol[indx,.]'~1e2*lowmat2(arraytomat(cor[indx,.,.]),error(0));
print ftosa(data,"%lf",3,0);

user_date = 20120301;
indx = indnv(user_date,Dates);
data = (1e4*Spread[indx,.])'~1e2*vol[indx,.]'~1e2*lowmat2(arraytomat(cor[indx,.,.]),error(0));
print ftosa(data,"%lf",3,0);

user_date = 20120531;
indx = indnv(user_date,Dates);
data = (1e4*Spread[indx,.])'~1e2*vol[indx,.]'~1e2*lowmat2(arraytomat(cor[indx,.,.]),error(0));
print ftosa(data,"%lf",3,0);

FileName = "Results//sovereign-bonds-vol";
save ^FileName = vol;

FileName = "Results//sovereign-bonds-rho";
save ^FileName = cor;
